/*!
        @file    threadManager_OpenMP.cpp

        @brief

        @author  Hideo Matsufuru (matsufuru)
                 $LastChangedBy: kanamori $

        @date    $LastChangedDate:: 2020-03-17 14:24:32 #$

        @version $LastChangedRevision: 2060 $
*/

#include "ResourceManager/threadManager_OpenMP.h"


#include "Communicator/communicator.h"
#include "IO/bridgeIO.h"
using Bridge::vout;

// These are stub routine when OpenMP is not used.
//                                          [11 Mar 2014 H.Matsufuru]
//====================================================================
// initialization of static member variables.

int ThreadManager_OpenMP::m_Nthread             = 0;
Bridge::VerboseLevel ThreadManager_OpenMP::m_vl = Bridge::CRUCIAL;
std::vector<double>  ThreadManager_OpenMP::m_darray_reduction(0);
std::vector<float>   ThreadManager_OpenMP::m_darray_reductionF(0);

//====================================================================
void ThreadManager_OpenMP::init(int Nthread)
{
  m_vl = CommonParameters::Vlevel();

  m_Nthread = 1;

  vout.general(m_vl, "ThreadManager_OpenMP being setup.\n");
  vout.general(m_vl, "  Number of thread = %d\n", m_Nthread);
  vout.general(m_vl, "  OpenMP is not used: stub implementation.\n");
}


//====================================================================
void ThreadManager_OpenMP::finalize()
{
  vout.paranoiac(m_vl, "Thread manager says good-bye.\n");
}


//====================================================================
int ThreadManager_OpenMP::get_num_threads()
{
  return 1;
}


//====================================================================
int ThreadManager_OpenMP::get_thread_id()
{
  return 0;
}


//====================================================================
void ThreadManager_OpenMP::wait()
{
}


//====================================================================
void ThreadManager_OpenMP::barrier(int Nthread)
{
}


//====================================================================
void ThreadManager_OpenMP::sync_barrier_all()
{
  Communicator::sync();
}

//====================================================================
void ThreadManager_OpenMP::reduce_sum_global(double* a,
                                             const int num,
                                             const int i_thread, const int Nthread)
{
  std::vector<double> sum(num);
  Communicator::reduce_sum(num, &sum[0], a, 0);
  for(int i=0; i<num; i++){
    a[i] = sum[i];
  }
}

//====================================================================
void ThreadManager_OpenMP::reduce_sum_global(float* a,
                                             const int num,
                                             const int i_thread, const int Nthread)
{
  std::vector<float> sum(num);
  Communicator::reduce_sum(num, &sum[0], a, 0);
  for(int i=0; i<num; i++){
    a[i] = sum[i];
  }
}


//====================================================================
void ThreadManager_OpenMP::reduce_sum_global(double& a,
                                             const int i_thread, const int Nthread)
{
  reduce_sum_global(&a, 1, i_thread, Nthread);
}


//====================================================================
void ThreadManager_OpenMP::reduce_sum_global(float& a,
                                             const int i_thread, const int Nthread)
{
  reduce_sum_global(&a, 1, i_thread, Nthread);
}


//====================================================================
void ThreadManager_OpenMP::assert_single_thread(
  const std::string& class_name)
{ // do nothing.
}


//====================================================================
//============================================================END=====
